/*=====[ The TesSeRact(TM) CXL User Interface Development System ]======*
 | Copyright (c) 1987-1991, Innovative Data Concepts, Inc.
 | All Rights Reserved.
 |
 | This Library is part of IDC's TesSeRact Development Tools product
 | line. For information about other IDC products, call 1-215-443-9705.
 *-V--------------------------------------------------------------------*
 | $Header:   tcxlmnu.h     552.2   26 Mar 1991 16:50:48  $
 |
 | $Log:   tcxlmnu.h    $
 *-D--------------------------------------------------------------------*
 | <TCXLmnu.h> : Definitions and prototypes for TCXL Menu system (User
 | level).
 *-N-----------------------[ Notes and Caveats ]------------------------*
 | 1) All Menu functions set TcxlErr and return negative ERR_??? values
 |    on errors.  Check return values before proceeding with processing a
 |    menu!
 | 2) The Mbld and Ibld Menu-build and Item-build data-objects are used
 |    to specify a complete menu-tree, its component menus and items to
 |    be built by MnuBuild(). A call to MnuBuild() with the requisite set
 |    of linked objects replaces the sequence of calls normally needed to
 |    define a complete menu tree and its menus and items, as given in
 |    4), which normally consists of multiple, nested calls to MnuDef()
 |    or MnuAct(), MnuTxt(), MnuItm(), ItmFun(), ItmTxt() and MnuEnd().
 |    This is a first step toward pre-compiled, tokenized menus.
 | 3) A menu tree defined by Mbld and Ibld objects is linked together by:
 |    a) MbldItm(m) points to the first element of an array of Ibld
 |       objects defining the items of a given menu.
 |    b) IbldSub(i) points to the Mbld object defining any child menu.
 | 3) Menus are defined as either horizontal, by MNU_HOR, vertical, by
 |    MNU_VER, or omnidirectional by MNU_OMN.
 | 4) Pop-up and pull-down menus are defined by the following sequence:
 |    a) Call MnuDef() or MnuAct() to begin the definition and give the
 |       number of items in the menu.
 |    b) Optionally call MnuTxt() to define text descriptions.
 |    c) Call MnuItm() for each item in the menu, along with optionally
 |       calling ItmFun() to assing "before" and/or "after" functions
 |       and/or ItmTxt() to define a text description for each item.
 |    d) Call MnuEnd() to complete the definition.
 |    Pop-up and Pull-down menus can be nested arbitrarily deep, limited
 |    only by memory requirements and may be nested in any way possible.
 | 5) A defined menu-tree, consisting of a set of menus is displayed and
 |    processed by calling MnuGet(). If user "before", "select" and/or
 |    "after" functions are defined for menu items within the tree, they
 |    can manipulate any item in the the displayed tree by defined tag-id,
 |    from the current menu to the root, by calling:
 |    a) ItmDis() to disable (make non-selectable) and redisplay an item.
 |    b) ItmEna() to enable a disabled item and redisplay it.
 |    c) ItmGet() to return a pointer to the item's selection text.
 |    d) ItmSet() to change the item's selection text.
 |    User functions can also manipulate the items in the current menu by
 |    defined tag-id by calling:
 |    a) ItmLoc() to return a pointer to the item's data-object.
 |    b) ItmNxt() to cause the specified item to be the next selected and
 |       move the selection bar to it (only useful from a "select"
 |       function).
 |    c) ItmDsp() to set the item's re-display flag.
 | 6) Hot-keys can select an item all the way to the root of the current
 |    menu-tree, with all child menus be closed along the way and the
 |    selected menu made current.
 | 7) The mouse can select an item in the immediate parent of the current
 |    menu, closing the current menu and making the parent current.
 | 8) A menu with no selectable items will be displayed with only [Esc]
 |    accepted as input if MNU_DSP is set. Otherwise it will result in an
 |    error-return.
 | 9) [Left]/[Right] arrow keys will select the previous/next pulldown
 |    menu if the current menu is a pulldown from a horizontal bar menu.
 |    Otherwise, they will select the previous/next item in either a
 |    horizontal or vertical menu.
 | 10) [Up]/[Down] arrow keys will select the previous/next item in
 |    either a horizontal or vertical menu.  In a horizontal bar menu,
 |    the [Down] key will select any attached pull-down menu.
 | 11) Text-descriptions, if used, are changed or erased as new items
 |    are selected. All items in a given menu need not have descriptions
 |    defined. If text-descriptions are not used, MbldDtr(m) MUST be set
 |    to 0xFF prior to calling MnuBuild.
 *======================================================================*/
#ifndef  _TCXLmnu_
#  define   _TCXLmnu_ 1
#  ifndef   _TCXLwin_
#    include <TCXLwin.h>               /* window definitions            */
#  endif
#  ifndef   _TCXLeng_
#    include <TCXLeng.h>               /* Page Selection Engine         */
#  endif
                              /*- Menu-item-feature mask bits ----------*/
#define  ITM_NOR     0x00     /* Normal (NONE of the following)         */
#define  ITM_PUL     0x01     /* Pull-down menu attached                */
#define  ITM_NSL     0x02     /* Not selectable                         */
#define  ITM_AFT     0x04     /* Close menu after select                */
#define  ITM_ALL     0x08     /* Close all menus when selected          */
#define  ITM_BEF     0x10     /* Close menu before select               */
                              /*- Menu types ---------------------------*/
#define  MNU_HOR     0x01     /* Horizontal                             */
#define  MNU_VER     0x02     /* Vertical                               */
#define  MNU_OMN     0x07     /* Omnidirectional                        */
#define  MNU_PUL     0x08     /* Pull-down                              */
#define  MNU_NQS     0x10     /* Disable quick selection                */
#define  MNU_SAV     0x20     /* Save last bar position                 */
#define  MNU_DSP     0x40     /* Display if no selectable items         */
#define  MNU_USE     0x80     /* Use current window                     */
                              /*- Internal MdfFlg bits -----------------*/
#define  MDF_USE     0x80     /* Use current window                     */
#define  MDF_TXT     0x01     /* Text-descriptions present              */
#define  MDF_TOP     0x10     /* Top of current menu chain              */
                              /*- Internal IdfFlg bits -----------------*/
#define  IDF_DSP     0x80     /* Redisplay pending                      */
#define  IDF_TXT     0x01     /* Text-description present               */

/*------------------[ Data objects and access macros ]------------------*/

TYP   struct   _Ibld  IbldT, *IbldP, **IbldI;
TYP   struct   _Idf  IdfT, *IdfP;      /* Object, pointer types         */
TYP   struct   _Mbld  MbldT, *MbldP, **MbldI;
TYP   struct   _Mctl MctlT;
TYP   struct   _Mdf  MdfT, *MdfP;

struct _Mbld                  /*-[Menu-build object ]-------------------*/
{  IbldP    mitm;             /* 00|00 : MbldItm : Item-build array     */
   IntT     mcnt;             /* 02|04 : MbldCnt : Number of items      */
   BytT     mbgr;             /* 04|06 : MbldBgR : Start screen row     */
   BytT     mbgc;             /* 05|07 : MbldBgC : Start screen col     */
   BytT     menr;             /* 06|08 : MbldEnR : End screen row       */
   BytT     menc;             /* 07|09 : MbldEnC : End screen col       */
   BytT     mtyp;             /* 08|0A : MbldTyp : Menu type            */
   BytT     mfrt;             /* 09|0B : MbldFrT : Frame type           */
   AtrT     mfra;             /* 0A|0C : MbldFrA : Frame attribute      */
   AtrT     mwat;             /* 0B|0D : MbldWat : Window attribute     */
   AtrT     mtat;             /* 0C|0E : MbldTat : Text attribute       */
   AtrT     msat;             /* 0D|0F : MbldSat : Select-char attr     */
   AtrT     mnat;             /* 0E|10 : MbldNat : Nonselectable attr   */
   AtrT     mbat;             /* 0F|11 : MbldBat : Selection bar attr   */
   TagT     mtag;             /* 10|12 : MbldTag : Initial item TagId   */
   VfvCP    mopn;             /* 12|14 : MbldOpn : "Open" function      */
   WrdT     musr;             /* 14|18 : MbldUsr : User menu-descriptor */
   BytT     mwid;             /* 16|1A : MbldWid : Selection bar width  */
   BytT     mofs;             /* 17|1B : MbldOfs : Text offset in bar   */
   BytT     mdtr;             /* 18|1C : MbldDtR : Text descr. row      */
   BytT     mdtc;             /* 19|1D : MbldDtC : Text descr. col      */
   AtrT     mdat;             /* 1A|1E : MbldDat : Text descr. attr     */
   BytT     mfut;             /* 1B|1F : MbldFut : Reserved for future  */
};                            /* 1C|20 ]--------------------------------*/
#define  MbldItm(m)  ((m)->mitm)                /* Item-build array     */
#define  MbldCnt(m)  ((m)->mcnt)                /* Number of items      */
#define  MbldBgR(m)  ((m)->mbgr)                /* Start screen row     */
#define  MbldBgC(m)  ((m)->mbgc)                /* Start screen col     */
#define  MbldEnR(m)  ((m)->menr)                /* End screen row       */
#define  MbldEnC(m)  ((m)->menc)                /* End screen col       */
#define  MbldTyp(m)  ((m)->mtyp)                /* Menu type            */
#define  MbldFrT(m)  ((m)->mfrt)                /* Frame type           */
#define  MbldFrA(m)  ((m)->mfra)                /* Frame attribute      */
#define  MbldWat(m)  ((m)->mwat)                /* Window attribute     */
#define  MbldTat(m)  ((m)->mtat)                /* Text attribute       */
#define  MbldSat(m)  ((m)->msat)                /* Select-char attr     */
#define  MbldNat(m)  ((m)->mnat)                /* Nonselectable attr   */
#define  MbldBat(m)  ((m)->mbat)                /* Selection bar attr   */
#define  MbldTag(m)  ((m)->mtag)                /* Initial item TagId   */
#define  MbldOpn(m)  ((m)->mopn)                /* "Open" function      */
#define  MbldUsr(m)  ((m)->musr)                /* User item-descriptor */
#define  MbldWid(m)  ((m)->mwid)                /* Selection bar width  */
#define  MbldOfs(m)  ((m)->mofs)                /* Text offset in bar   */
#define  MbldDtR(m)  ((m)->mdtr)                /* Text descr. row      */
#define  MbldDtC(m)  ((m)->mdtc)                /* Text descr. col      */
#define  MbldDat(m)  ((m)->mdat)                /* Text descr. attr     */

struct   _Ibld                /*-[ Item-build object ]------------------*/
{  ChrP     istr;             /* 00|00 : IbldStr : Selection text       */
   ChrT     ichr;             /* 02|04 : IbldChr : Selection character  */
   BytT     imsk;             /* 03|05 : IbldMsk : Feature mask         */
   BytT     irow;             /* 04|06 : IbldRow : Window row           */
   BytT     icol;             /* 05|07 : IbldCol : Window column        */
   TagT     itag;             /* 06|08 : IbldTag : Item TagId           */
   TagT     ihlp;             /* 08|0A : IbldHlp : Help TagId           */
   KeyT     ikey;             /* 0A|0C : IbldKey : Hot key              */
   WrdT     iusr;             /* 0C|0E : IbldUsr : User item-descriptor */
   VfvCP    ibef;             /* 0E|10 : IbldBef : "Before" function    */
   VfvCP    isel;             /* 10|14 : IbldSel : "Select" function    */
   VfvCP    iaft;             /* 12|18 : IbldAft : "After" function     */
   ChrP     itxt;             /* 14|1C : IbldTxt : Description text     */
   MbldP    isub;             /* 16|20 : IbldSub : Child menu-build     */
};                            /* 18|24 ]--------------------------------*/
#define  IbldStr(i)  ((i)->istr)                /* Selection text       */
#define  IbldChr(i)  ((i)->ichr)                /* Selection character  */
#define  IbldMsk(i)  ((i)->imsk)                /* Feature mask         */
#define  IbldRow(i)  ((i)->irow)                /* Window row           */
#define  IbldCol(i)  ((i)->icol)                /* Window column        */
#define  IbldTag(i)  ((i)->itag)                /* Item tag id          */
#define  IbldHlp(i)  ((i)->ihlp)                /* Help tag-id          */
#define  IbldKey(i)  ((i)->ikey)                /* Hot key              */
#define  IbldUsr(i)  ((i)->iusr)                /* User item-descriptor */
#define  IbldBef(i)  ((i)->ibef)                /* "Before" function    */
#define  IbldAft(i)  ((i)->iaft)                /* "After" function     */
#define  IbldSel(i)  ((i)->isel)                /* "Select" function    */
#define  IbldTxt(i)  ((i)->itxt)                /* Description text     */
#define  IbldSub(i)  ((i)->isub)                /* Child menu-build     */

struct   _Mctl                /*-[ Menu-control object ]----------------*/
{  MdfP     mtop;             /* 00|00 : MctlTop : Root menu            */
   MdfP     mcur;             /* 02|04 : MctlCur : Current menu         */
   IntT     mmnu;             /* 04|08 : MctlMnu : Menu level           */
   IntT     mitm;             /* 06|0A : MctlItm : Item level           */
   TagT     mtag;             /* 08|0C : MctlTag : Selected TagId       */
};                            /* 0A|0E ]--------------------------------*/

GBL   MctlT CDC   _MnuCtl;    /*- global menu-control object -----------*/
#define  MctlTop  (_MnuCtl.mtop)                /* Root menu            */
#define  MctlCur  (_MnuCtl.mcur)                /* Current menu         */
#define  MctlMnu  (_MnuCtl.mmnu)                /* Menu level           */
#define  MctlItm  (_MnuCtl.mitm)                /* Item level           */
#define  MctlTag  (_MnuCtl.mtag)                /* Selected TagId       */

struct   _Mdf                 /*-[ Menu-definition object ]-------------*/
{  MdfP     mprv;             /* 00|00 : MdfPrv : Previous/parent menu  */
   IdfP     mtop;             /* 02|04 : MdfTop : Item-list head        */
   IdfP     mbot;             /* 04|08 : MdfBot : Item-list tail        */
   IdfP     mcur;             /* 06|0C : MdfCur : Current item          */
   ElstP    meng;             /* 08|10 : MdfEng : Page Selection Engine */
   VfvCP    mopn;             /* 0A|14 : MdfOpn : User "open" function  */
   TagT     mtag;             /* 0C|18 : MdfTag : Selected TagId        */
   VposT    mbeg;             /* 0E|1A : MdfBeg : Window start position */
   VposT    mend;             /* 10|1C : MdfEnd : Window end position   */
   VposT    mpos;             /* 12|1E : MdfPos : Item start position   */
   VposT    mdsc;             /* 14|20 : MdfDsc : Description/help pos  */
   VposT    mlim;             /* 16|22 : MdfLim : Image size/limits     */
   BoxT     mfrm;             /* 18|24 : MdfFrm : Frame                 */
   BytT     mtyp;             /* 1A|26 : MdfTyp : Type mask             */
   BytT     mwid;             /* 1B|27 : MdfWid : Bar width             */
   BytT     mofs;             /* 1C|28 : MdfOfs : Text offset           */
   AtrT     mwat;             /* 1D|29 : MdfWat : Window attribute      */
   AtrT     mtat;             /* 1E|2A : MdfTat : Text attribute        */
   AtrT     msat;             /* 1F|2B : MdfSat : Select-char attribute */
   AtrT     mnat;             /* 20|2C : MdfNat : Non-select attribute  */
   AtrT     mbat;             /* 21|2D : MdfBat : Bar attribute         */
   AtrT     mdat;             /* 22|2E : MdfDat : Description/help attr */
   BytT     mflg;             /* 23|2F : MdfFlg : Internal flags        */
   WrdT     musr;             /* 24|30 : MdfUsr : User menu descriptor  */
};                            /* 26|32 ]--------------------------------*/
#define  MdfPrv(m)   ((m)->mprv)                /* Previous menu        */
#define  MdfTop(m)   ((m)->mtop)                /* Item-list head       */
#define  MdfBot(m)   ((m)->mbot)                /* Item-list tail       */
#define  MdfCur(m)   ((m)->mcur)                /* Current item         */
#define  MdfEng(m)   ((m)->meng)                /* Page Engine          */
#define  MdfOpn(m)   ((m)->mopn)                /* "Open" function      */
#define  MdfTag(m)   ((m)->mtag)                /* Selected item TagId  */
#define  MdfBeg(m)   ((m)->mbeg)                /* Start position       */
#define  MdfBegW(m)  (VposW(MdfBeg(m)))            /* Word              */
#define  MdfBegC(m)  (VposC(MdfBeg(m)))            /* Col               */
#define  MdfBegR(m)  (VposR(MdfBeg(m)))            /* Row               */
#define  MdfEnd(m)   ((m)->mend)                /* End position         */
#define  MdfEndW(m)  (VposW(MdfEnd(m)))            /* Word              */
#define  MdfEndC(m)  (VposC(MdfEnd(m)))            /* Col               */
#define  MdfEndR(m)  (VposR(MdfEnd(m)))            /* Row               */
#define  MdfPos(m)   ((m)->mpos)                /* Item start position  */
#define  MdfPosW(m)  (VposW(MdfPos(m)))            /* Word              */
#define  MdfPosC(m)  (VposC(MdfPos(m)))            /* Col               */
#define  MdfPosR(m)  (VposR(MdfPos(m)))            /* Row               */
#define  MdfDsc(m)   ((m)->mdsc)                /* Description/help pos */
#define  MdfDscW(m)  (VposW(MdfDsc(m)))            /* Word              */
#define  MdfDscC(m)  (VposC(MdfDsc(m)))            /* Col               */
#define  MdfDscR(m)  (VposR(MdfDsc(m)))            /* Row               */
#define  MdfLim(m)   ((m)->mlim)                /* Image size/limits    */
#define  MdfLimW(m)  (VposW(MdfLim(m)))            /* Word              */
#define  MdfLimC(m)  (VposC(MdfLim(m)))            /* Cols              */
#define  MdfLimR(m)  (VposR(MdfLim(m)))            /* Rows              */
#define  MdfFrm(m)   ((m)->mfrm)                /* Frame                */
#define  MdfFrmT(m)  (BoxTyp(MdfFrm(m)))           /* Type              */
#define  MdfFrmA(m)  (BoxAtr(MdfFrm(m)))           /* Attr              */
#define  MdfTyp(m)   ((m)->mtyp)                /* Type mask            */
#define  MdfHor(m)   (0 != (MdfTyp(m) & MNU_HOR))  /* Horizontal        */
#define  MdfVer(m)   (0 != (MdfTyp(m) & MNU_VER))  /* Vertical          */
#define  MdfOmn(m)   (MNU_OMN==(MdfTyp(m)&MNU_OMN))/* Omnidirectional   */
#define  MdfPul(m)   (0 != (MdfTyp(m) & MNU_PUL))  /* Pull-down         */
#define  MdfNqs(m)   (0 != (MdfTyp(m) & MNU_NQS))  /* No quick select   */
#define  MdfSav(m)   (0 != (MdfTyp(m) & MNU_SAV))  /* Save bar position */
#define  MdfDsp(m)   (0 != (MdfTyp(m) & MNU_DSP))  /* Always display    */
#define  MdfWid(m)   ((m)->mwid)                /* Bar width            */
#define  MdfOfs(m)   ((m)->mofs)                /* Text offset          */
#define  MdfWat(m)   ((m)->mwat)                /* Window attr          */
#define  MdfTat(m)   ((m)->mtat)                /* Text attribute       */
#define  MdfSat(m)   ((m)->msat)                /* Select char attr     */
#define  MdfNat(m)   ((m)->mnat)                /* Non-select attr      */
#define  MdfBat(m)   ((m)->mbat)                /* Bar attribute        */
#define  MdfDat(m)   ((m)->mdat)                /* Description/help attr*/
#define  MdfFlg(m)   ((m)->mflg)                /* Internal flags       */
#define  MdfFuse(m)  (0 != (MdfFlg(m) & MDF_USE))  /* Current window    */
#define  MdfFtxt(m)  (0 != (MdfFlg(m) & MDF_TXT))  /* Descriptions      */
#define  MdfFtop(m)  (0 != (MdfFlg(m) & MDF_TOP))  /* Top of current    */
#define  MdfUsr(m)   ((m)->musr)                /* User descriptor      */

struct  _Idf                  /*-[ Item-definition object ]-------------*/
{  MdfP  ipar;                /* 00|00 : IdfPar : Parent menu           */
   MdfP  isub;                /* 02|04 : IdfSub : Child menu            */
   ChrP  istr;                /* 04|08 : IdfStr : Selection text        */
   ChrP  itxt;                /* 06|0C : IdfTxt : Description/help text */
   VfvCP isel;                /* 08|10 : IdfSel : "Select" function     */
   VfvCP ibef;                /* 0A|14 : IdfBef : "Before" function     */
   VfvCP iaft;                /* 0C|18 : IdfAft : "After"  function     */
   KeyT  ikey;                /* 0E|1C : IdfKey : Hot key               */
   TagT  itag;                /* 10|1E : IdfTag : Item TagId            */
   TagT  ihlp;                /* 12|20 : IdfHlp : Help TagId            */
   VposT ispo;                /* 14|22 : IdfSpo : Select text position  */
   BytT  ichr;                /* 16|24 : IdfChr : Quick select char     */
   BytT  imsk;                /* 17|25 : IdfMsk : Feature mask          */
   BytT  ilen;                /* 18|26 : IdfLen : Selection-text length */
   BytT  iflg;                /* 19|27 : IdfFlg : Internal flags        */
   WrdT  iusr;                /* 1A|28 : IdfUsr : User item descriptor  */
};                            /* 1C|2A ]--------------------------------*/
#define  IdfPar(i)   ((i)->ipar)                /* Parent menu          */
#define  IdfSub(i)   ((i)->isub)                /* Child menu           */
#define  IdfStr(i)   ((i)->istr)                /* Selection text       */
#define  IdfTxt(i)   ((i)->itxt)                /* Description text     */
#define  IdfSel(i)   ((i)->isel)                /* "Select" function    */
#define  IdfBef(i)   ((i)->ibef)                /* "Before" function    */
#define  IdfAft(i)   ((i)->iaft)                /* "After"  function    */
#define  IdfKey(i)   ((i)->ikey)                /* Hot key              */
#define  IdfTag(i)   ((i)->itag)                /* Tag ID               */
#define  IdfHlp(i)   ((i)->ihlp)                /* Help tag             */
#define  IdfPos(i)   ((i)->ispo)                /* Text pos             */
#define  IdfPosW(i)  (VposW(IdfPos(i)))            /* Word              */
#define  IdfPosC(i)  (VposC(IdfPos(i)))            /* Col               */
#define  IdfPosR(i)  (VposR(IdfPos(i)))            /* Row               */
#define  IdfChr(i)   ((i)->ichr)                /* Quick select char    */
#define  IdfMsk(i)   ((i)->imsk)                /* Feature mask         */
#define  IdfNor(i)   (IdfMsk(i) == ITM_NOR))       /* Normal            */
#define  IdfPul(i)   (0 != (IdfMsk(i) & ITM_PUL))  /* Pull-down         */
#define  IdfNsl(i)   (0 != (IdfMsk(i) & ITM_NSL))  /* Not selectable    */
#define  IdfCaf(i)   (0 != (IdfMsk(i) & ITM_AFT))  /* Close after       */
#define  IdfCal(i)   (0 != (IdfMsk(i) & ITM_ALL))  /* Close all         */
#define  IdfCbf(i)   (0 != (IdfMsk(i) & ITM_BEF))  /* Close before      */
#define  IdfLen(i)   ((i)->ilen)                /* Text length          */
#define  IdfFlg(i)   ((i)->iflg)                /* Internal flags       */
#define  IdfFdsp(i)  (0 != (IdfFlg(i) & IDF_DSP))  /* Redisplay         */
#define  IdfFtxt(i)  (0 != (IdfFlg(i) & IDF_TXT))  /* Description       */
#define  IdfUsr(i)   ((i)->iusr)                /* User descriptor      */

/*------------------------[ Function prototypes ]-----------------------*/

#ifdef __cplusplus                     /* no mangling, please           */
   extern "C" {
#endif
                              /*- Build complete menu tree -------------*/
IntT  CTYP  MnuBuild(MbldP m);         /* Build current menu tree       */
                              /*- Menu and item definition -------------*/
IntT  CTYP  MnuAct(IntT ni);           /* Define menu in active window  */
IntT        MnuBeg(IntT sr, IntT sc,   /* obsolete                      */
   IntT er, IntT ec, BytT bt, AtrT ba,
   AtrT wa, VfvCP fo);
IntT  CTYP  MnuDef(IntT sr, IntT sc,   /* Define menu                   */
   IntT er, IntT ec, BytT bt, AtrT ba,
   AtrT wa, VfvCP fo, IntT ni);
IntT  CTYP  MnuTxt(BytT wr, BytT wc,   /* Define menu text-descriptions */
   AtrT  at);
IntT  CTYP  MnuItm(BytT wr, BytT wc,   /* Define menu item              */
   ChrP ps, ChrT sc, TagT id, BytT fm,
   VfvCP sf, KeyT kc, TagT ht);
IntT  CTYP  ItmFun(VfvCP bf,           /* Assign "before" and "after"   */
   VfvCP af);                          /*    functions to item          */
IntT  CTYP  ItmTxt(ChrP ps);           /* Assign item text description  */
IntT  CTYP  MnuEnd(TagT id, BytT mt,   /* End menu definition           */
   BytT bw, BytT tp, AtrT ta,
   AtrT sa, AtrT na, AtrT ba);
                              /*- Menu input processing ----------------*/
MdfP        MnuCur(NOARG);             /* Current menu object pointer   */
TagT  CTYP  MnuGet(NOARG);             /* Display/process menu tree     */
TagT        MnuShow(NOARG);            /* obsolete                      */
                              /*- Current-menu item-processing ---------*/
IdfP        ItmCur(NOARG);             /* Current item object pointer   */
IdfP  CTYP  ItmLoc(TagT id);           /* Find item <id>                */
IntT  CTYP  ItmNxt(TagT id);           /* Make item <id> current        */
IntT  CTYP  ItmDsp(TagT id);           /* Flag item <id> for redisplay  */
                              /*- Global item-processing ---------------*/
IntT  CTYP  ItmDis(TagT id);           /* Disable item <id>             */
IntT  CTYP  ItmEna(TagT id);           /* Enable item <id>              */
ChrP  CTYP  ItmGet(TagT id);           /* Get item <id> select text     */
IntT  CTYP  ItmSet(TagT id, ChrP tp,   /* Set item <id> select text,    */
   FlgT df);                           /*    optionally re-display      */
                              /*- Internals ----------------------------*/
IdfP  CTYP  _ItmKey(KeyT kc);
IdfP  CTYP  _ItmPtr(TagT id);
VOID  CTYP  _ItmPut(IdfP ip, FlgT hb);
                              /*- Implementation macros ----------------*/
#define  MnuBeg(sr,sc,er,ec,bt,ba,wa,fo) \
   MnuBeg() not supported in TCXL 5.52 -- use MnuDef() instead!
#define  MnuCur()    (MctlCur)
#define  MnuShow()   MnuGet()          /* ... alias for 5.51            */
#define  ItmCur()    (MdfCur(MctlCur))
#define  IdfSpo(i)   (IdfPos(i))
#define  IdfSpoW(i)  (IdfPosW(i))
#define  IdfSpoC(i)  (IdfPosC(i))
#define  IdfSpoR(i)  (IdfPosR(i))
#ifdef __cplusplus
   }
#endif
#endif   /*- _TCXLmnu_ : End of TCXLmnu.h ------------------------------*/
