/*=====[ The TesSeRact(TM) CXL User Interface Development System ]======*
 | Copyright (c) 1987-1991, Innovative Data Concepts, Inc.
 | All Rights Reserved.
 |
 | This Library is part of IDC's TesSeRact Development Tools product
 | line. For information about other IDC products, call 1-215-443-9705.
 *-V--------------------------------------------------------------------*
 | $Header:   tcxlmou.h     552.4   31 May 1991 14:18:12  $
 |
 | $Log:   tcxlmou.h    $
 *-D--------------------------------------------------------------------*
 | <TCXLmou.h> : Definitions and prototypes for TCXL Micro$oft-compatible
 | mouse support functions.
 *-N-----------------------[ Notes and Caveats ]------------------------*
 | 1) All mouse positions are in text rows and columns.
 | 2) _MSinit() and _MSterm() are internal and MUST NOT be called by user
 |    functions!  Call TcxlInit() to initialize TCXL, NOT the subsystem
 |    initializers.
 | 3) _MsHide()/_MSshow() only hide/show the mouse-cursor ONE level deep.
 | 4) MsDisab()/MSenab() disable/enable mouse-support once initialized by
 |    _MSinit().
 | 5) MSupdat() operates without coupling to the normal hide/show logic.
 |    MSupdat(TRUE, ...) MUST be called after MSupdat(FALSE, ...) to
 |    ensure the cursor is not hidden until a MSreset() call.
 | 6) MSreset() resets the mouse hardware and software, maintaining the
 |    current mouse-state.
 | 7) See <TCXLcur.h> for descriptions of mouse-support and the Mou????()
 |    functions.
 *======================================================================*/
#ifndef  _TCXLmou_
#  define   _TCXLmou_ 1                /* Only once!                    */
#  ifndef   _TCXLdef_
#     include  <TCXLdef.h>             /* standard definitions          */
#  endif
                                       /*- Mouse support levels---------*/
#define  MOU_NONE 0x00                 /* No mouse support              */
#define  MOU_KEYS 0x01                 /* Emulate arrow keys            */
#define  MOU_CURS 0x02                 /* Free-moving visible cursor    */
#define  MOU_FULL 0x03                 /* Full mouse support            */
                                       /*- Mouse button indices --------*/
#define  MOU_LBTN 0x01                 /* Left button                   */
#define  MOU_RBTN 0x02                 /* Right button                  */
#define  MOU_MBTN 0x04                 /* Middle button                 */
                                       /*- Mouse control-flags ---------*/
#define  MOU_INIT 0x80                 /* Mouse initialized             */
#define  MOU_3BTN 0x40                 /* 3-button mouse                */
#define  MOU_ENAB 0x20                 /* Mouse enabled                 */
#define  MOU_HDWC 0x10                 /* Hardware text-cursor          */
#define  MOU_VISI 0x08                 /* Visible cursor                */
#define  MOU_USER 0x03                 /* User max support-level        */

/*------------------[ Data objects and access macros ]------------------*/

TYP   struct   _Mou  MouT;

struct   _Mou                 /*-[ Mouse-control object ]---------------*/
{  BytT  mlvl;                /* 00 : MouLvl  : Current support-level   */
   BytT  mcol;                /* 01 : MouCol  : Current text column     */
   BytT  mrow;                /* 02 : MouRow  : Current text row        */
   BytT  mstk[45];            /* 03 : MouStk  : 15-level state stack    */
   BytT  mdep;                /* 30 : MouDep  : Stack depth             */
   BytT  mcnd;                /* 31 : MouCnd  : Conditional-hide count  */
   BytT  mflg;                /* 32 : MouFlg  : Control-flags           */
   BytT  mbtn;                /* 33 : MouBtn  : Button-status bits      */
   IntT  mcnt;                /* 34 : MouCnt  : Button-count            */
   IntT  mhor;                /* 36 : MouHor  : Horizontal position     */
   IntT  mver;                /* 38 : MouVer  : Vertical position       */
   KeyT  mlkc;                /* 3A : MouLkey : Left-button keycode     */
   KeyT  mrkc;                /* 3C : MouRkey : Right-button keycode    */
   KeyT  mmkc;                /* 3E : MouMkey : Middle-button keycode   */
   WrdT  mand;                /* 40 : MouAnd  : AND-mask/start-line     */
   WrdT  mxor;                /* 42 : MouXor  : XOR-mask/stop-line      */
   WrdT  musr;                /* 44 : MouUsr  : User-defined Mouse mask */
   BytT  msco;                /* 46 : MouColS : Saved text column       */
   BytT  msro;                /* 47 : MouRowS : Saved text row          */
   BytT  msbu;                /* 48 : MouBtnS : Saved button-status     */
   BytT  mdel;                /* 49 : MouDel  : Response delay count    */
};                            /* 4A ]-----------------------------------*/

GBL   MouT  CDC   _MouCtl;    /*- Global mouse-control object ----------*/

#define  MouLvl   (_MouCtl.mlvl)                /* Current level        */
#define  MouNone  (MOU_NONE == MouLvl)             /* None              */
#define  MouKeys  (MOU_KEYS == (MouLvl&MOU_USER))  /* Arrow-emulation   */
#define  MouCurs  (0 != (MouLvl & MOU_CURS))       /* Free-moving       */
#define  MouFull  (MOU_FULL == (MouLvl&MOU_USER))  /* Full              */
#define  MouCol   (_MouCtl.mcol)                /* Current column       */
#define  MouRow   (_MouCtl.mrow)                /* Current row          */
#define  MouStk   (_MouCtl.mstk)                /* 15-level stack       */
#define  MouDep   (_MouCtl.mdep)                /* Stack depth          */
#define  MouCnd   (_MouCtl.mcnd)                /* Conditional-hide     */
#define  MouFlg   (_MouCtl.mflg)                /* Control flags        */
#define  MouInit  (0 != (MouFlg & MOU_INIT))       /* Initialized       */
#define  Mou3btn  (0 != (MouFlg & MOU_3BTN))       /* 3-button          */
#define  MouEnab  (0 != (MouFlg & MOU_ENAB))       /* Enabled           */
#define  MouDisa  (0 == (MouFlg & MOU_ENAB))       /* Disabled          */
#define  MouHdwC  (0 != (MouFlg & MOU_HDWC))       /* Hardware cursor   */
#define  MouAtrC  (0 == (MouFlg & MOU_HDWC))       /* Attribute cursor  */
#define  MouVisi  (0 != (MouFlg & MOU_VISI))       /* Visible cursor    */
#define  MouHidd  (0 == (MouFlg & MOU_VISI))       /* Hidden cursor     */
#define  MouBtn   (_MouCtl.mbtn)                /* Button-status bits   */
#define  MouLbtn  (0 != (MouBtn & 0x01))           /* Left down         */
#define  MouRbtn  (0 != (MouBtn & 0x02))           /* Right down        */
#define  MouMbtn  (0 != (MouBtn & 0x04))           /* Middle down       */
#define  MouHor   (_MouCtl.mhor)                /* Horizontal position  */
#define  MouVer   (_MouCtl.mver)                /* Vertical position    */
#define  MouLkey  (_MouCtl.mlkc)                /* Left keycode         */
#define  MouRkey  (_MouCtl.mrkc)                /* Right keycode        */
#define  MouMkey  (_MouCtl.mmkc)                /* Middle keycode       */
#define  MouAnd   (_MouCtl.mand)                /* Cursor AND-mask      */
#define  MouXor   (_MouCtl.mxor)                /* Cursor XOR-mask      */
#define  MouUsr   (_MouCtl.musr)                /* User max support     */
#define  MouUsrMnu (MouUsr & MOU_USER)             /* menu system       */
#define  MouUsrEnt ((MouUsr >> 2) & MOU_USER)      /* entry system      */
#define  MouUsrSel ((MouUsr >> 4) & MOU_USER)      /* selection system  */
#define  MouUsrHlp ((MouUsr >> 6) & MOU_USER)      /* help system       */
#define  MouBtnS  (_MouCtl.msbu)                /* Saved Button-status  */
#define  MouLbtnS (0 != (MouBtnS & 0x01))          /* Left down         */
#define  MouRbtnS (0 != (MouBtnS & 0x02))          /* Right down        */
#define  MouMbtnS (0 != (MouBtnS & 0x04))          /* Middle down       */
#define  MouColS  (_MouCtl.msco)                /* Saved column         */
#define  MouRowS  (_MouCtl.msro)                /* Saved row            */
#define  MouDel   (_MouCtl.mdel)                /* Response delay count */

/*-------------------------[ function prototypes ]----------------------*/

#ifdef __cplusplus                     /* No mangling, please           */
   extern "C" {
#endif
VOID  PAS   MSclear(NOARG);            /* Clear all button counts       */
VOID  PAS   MScurs(WrdT ct, WrdT sm,   /* Define text cursor            */
   WrdT cm);
VOID        MSdelay(BytT d);           /* Assign new response-delay     */
VOID        MSdisab(NOARG);            /* Disable mouse                 */
VOID        MSenab(NOARG);             /* Enable mouse, if initialized  */
VOID  PAS   MSgoto(IntT r, IntT c);    /* Position cursor to <r,c>      */
VOID  PAS   MShome(NOARG);             /* Home cursor, clear all counts */
VOID  PAS   MSkeys(KeyT l, KeyT r,     /* Assign left <l>, right <r>,   */
   KeyT m);                            /*    middle <m> button keycodes */
VOID  PAS   MSlimit(IntT sr, IntT sc,  /* Define screen limits          */
   IntT er, IntT ec);
VOID  PAS   MSmove(NOARG);             /* Read motion values            */
VOID  PAS   MSpress(IntT b);           /* Get button <b> press count    */
VOID  PAS   MSreles(IntT b);           /* Get button <b> release count  */
VOID  PAS   MSreset(NOARG);            /* Reset, maintaining state      */
VOID  PAS   MSspeed(IntT x, IntT y);   /* Set <x,y> speed               */
VOID  PAS   MSstat(NOARG);             /* Get status, position          */
FlgT  PAS   MSupdat(FlgT hs, IntT sr,  /* Conditionlly hide/show cursor */
   IntT sc, IntT er, IntT ec);         /*    for update of screen region*/
                              /*- Set maximum user-level mouse-support -*/
VOID        MSSetEnt(BytT x);          /* Entry system                  */
VOID        MSSetHlp(BytT x);          /* Help system                   */
VOID        MSSetMnu(BytT x);          /* Menu system                   */
VOID        MSSetSel(BytT x);          /* Selection system              */
                              /*- Internals ----------------------------*/
VOID  PAS   _MShide(NOARG);            /* Hide cursor if shown          */
IntT  PAS   _MSinit(NOARG);            /* Detect, initialize mouse      */
VOID  PAS   _MSpop(NOARG);             /* Pop mouse-state               */
VOID  PAS   _MSpush(BytT s);           /* Push mouse-state              */
VOID  PAS   _MSshow(NOARG);            /* Show cursor if hidden         */
VOID  PAS   _MSterm(NOARG);            /* Terminate support             */
VOID  PAS   _MSwait(NOARG);            /* Wait for button-release       */
                              /*- Implementation macros ----------------*/
#define  MSdelay(d)  if(MouInit) MouDel = (d)
#define  MSdisab()   (MouFlg &= !MOU_ENAB)
#define  MSenab()    if(MouInit) MouFlg |= MOU_ENAB
#define  MSSetMnu(x) (MouUsr = (MouUsr & 0xFC) | (x & MOU_USER))
#define  MSSetEnt(x) (MouUsr = (MouUsr & 0xF3) | ((x & MOU_USER) << 2))
#define  MSSetSel(x) (MouUsr = (MouUsr & 0xCF) | ((x & MOU_USER) << 4))
#define  MSSetHlp(x) (MouUsr = (MouUsr & 0x3F) | ((x & MOU_USER) << 6))
#ifdef __cplusplus
   }
#endif
#endif   /*- _TCXLmou_ : End of TCXLmou.h ------------------------------*/
