/*=====[ The TesSeRact(TM) CXL User Interface Development System ]======*
 | Copyright (c) 1987-1991, Innovative Data Concepts, Inc.
 | All Rights Reserved.
 |
 | This Library is part of IDC's TesSeRact Development Tools product
 | line. For information about other IDC products, call 1-215-443-9705.
 *-V--------------------------------------------------------------------*
 | $Header:   tcxlnls.h     552.2   05 Apr 1991 15:05:44  $
 |
 | $Log:   tcxlnls.h    $
 *-D--------------------------------------------------------------------*
 | <TCXLnls.h> : Definitions and prototypes for TCXL national-language
 | support [NLS] sub-system.
 *-N-----------------------[ Notes and Caveats ]------------------------*
 | 1) Included by <TCXLfmt.h>
 | 2) NLS defaults to US/CDN (English), however, if DOS 3.3 extended
 |    country support indicates a different country code, the current
 |    country information is used.
 | 3) This implementation REQUIRES MS/PC-DOS 3.30 or higher and will
 |    exit with an error at a lower DOS revision or if a DOS error
 |    occurs during startup.
 | 3) _NlsInit() is INTERNAL and NOT intended to be called by user code!
 |    Call TcxlInit() to initialize TCXL, NOT any of the sub-system
 |    initializers!
 | 4) The NlsIs???() and NlsTo???() functions map extended ASCII (above
 |    0x7F ONLY. They are only searched after an ASCII isupper(), etc.,
 |    fails and are used to:
 |    a) translate characters between cases
 |    b) determine if a character is alphabetic or printable.
 | 5) The NLS case-map table is 256 bytes long. It contains 128 pairs of
 |    chars, which map any country-dependent upper/lower case non-ASCII
 |    characters above 0x7F, [0x80..0xFF]. Each pair consists of an
 |    uppercase, then a lowercase char, if any exist for the value of the
 |    pair, otherwise both chars are 0x00.  It is searched AFTER an ASCII
 |    isupper(), etc. fails. _NlsInit() builds the table from the UCase
 |    table returned as part of the DOS Extended Country information.
 | 6) The NLS print-map is a NUL-terminated string of non-alphabetic,
 |    non-ASCII chars above 0x7F that are to be cosidered printable
 |    (punctuation, diacriticals, currency symbols, etc.). It is serached
 |    AFTER the complete case-map by NlsIsPrn().  MLS_ML_PRN defines a
 |    print-map for the IBM Multi-lingual code-page (850). It (or any
 |    user-defined, NUL-terminated print-map string of chars) may be
 |    assigned in place of the default US/CDN print-map any time after
 |    TcxlInit() is called via the NlsPrn access-macro. The default map
 |    contains: "\x9B\x9C\x9D\x9E\x9F\xA7\xAD\x00".
 | 7) _NlsCtl is declared in _NlsInit.C
 *======================================================================*/
#ifndef  _TCXLnls_
#  define   _TCXLnls_ 1                /* Only once!                    */
#  ifndef   _TCXLdef_
#     include  <TCXLdef.h>             /* Standard definitions          */
#  endif
#define  NLS_ML_PRN "\x9C\x9E\x9F\xA8\xAD\xAE\xAF\xBD\xBE\xCF\xDD\xEE"  \
                    "\xEF\xF0\xF4\xF5\xF6\xF7\xF8\xF9\xFA\x00"

/*------------------[ Data objects and access macros ]------------------*/

TYP   struct   _Nls  NlsT, *NlsP;
struct   _Nls                 /*-[ National-language object ]-----------*/
{  WrdT     npag;             /* 00|00 : NlsPag : Global code-page      */
   WrdT     ncty;             /* 02|02 : NlsCty : Country-code          */
   ChrP     nsym;             /* 04|04 : NlsSym : Currency-symbol       */
   BytT     nsyl;             /* 06|08 : NlsSyl :    Length             */
   ChrT     ndsc;             /* 07|09 : NlsDsc : Decimal separator     */
   ChrT     ntsc;             /* 08|0A : NlsTsc : Thousands separator   */
   ChrT     ndtc;             /* 09|0B : NlsDtc : Date separator        */
   ChrT     ntmc;             /* 0A|0C : NlsTmc : Time separator        */
   BytT     nndp;             /* 0B|0D : NlsNdp : Decimal places        */
   ChrT     nlsc;             /* 0C|0E : NlsLsc : List Separator        */
   ChrT     nyes;             /* 0D|0F : NlsYes : Uppercase 'Yes' char  */
   ChrT     nno;              /* 0E|10 : NlsNo  : Uppercase 'No' char   */
   BytT     nflg;             /* 0F|11 : NlsFlg : Control flags         */
   ChrP     nupl;             /* 10|12 : NlsUpl : Extended case-map     */
   ChrP     nprn;             /* 12|16 : NlsPrn : Extended print-map    */
};                            /* 14|1A ]--------------------------------*/

GBL   NlsT  CDC   _NlsCtl;    /*- Global National-language object ------*/
#define  NlsPag   (_NlsCtl.npag)                /* Global code-page     */
#define  NlsCty   (_NlsCtl.ncty)                /* Country-code         */
#define  NlsSym   (_NlsCtl.nsym)                /* Currency symbol      */
#define  NlsSyl   (_NlsCtl.nsyl)                /*    Length            */
#define  NlsDsc   (_NlsCtl.ndsc)                /* Decimal separator    */
#define  NlsTsc   (_NlsCtl.ntsc)                /* Thousands separator  */
#define  NlsDtc   (_NlsCtl.ndtc)                /* Date separator       */
#define  NlsTmc   (_NlsCtl.ntmc)                /* Time separator       */
#define  NlsNdp   (_NlsCtl.nndp)                /* Decimal places       */
#define  NlsLsc   (_NlsCtl.nlsc)                /* List separator       */
#define  NlsYes   (_NlsCtl.nyes)                /* Uppercase 'Yes' char */
#define  NlsNo    (_NlsCtl.nno)                 /* Uppercase 'No' char  */
#define  NlsFlg   (_NlsCtl.nflg)                /* Control flags        */
#define  NlsCpf   (0 == (NlsFlg & 0x03))           /* Currency prefix   */
#define  NlsCsf   (0 != (NlsFlg & 0x01))           /* Currency suffix   */
#define  NlsCdp   (0 != (NlsFlg & 0x02))           /* Currency decimal  */
#define  NlsMdy   (0 == (NlsFlg & 0x0C))           /* Date mm dd yy     */
#define  NlsDmy   (0 != (NlsFlg & 0x04))           /* Date dd mm yy     */
#define  NlsYmd   (0 != (NlsFlg & 0x08))           /* Date yy mm dd     */
#define  Nls12h   (0 == (NlsFlg & 0x10))           /* Time 12-hour      */
#define  Nls24h   (0 != (NlsFlg & 0x10))           /* Time 24-hour      */
#define  NlsUpl   (_NlsCtl.nupl)                /* Extended case-map    */
#define  NlsPrn   (_NlsCtl.nprn)                /* Extended print-map   */

/*------------------------[ Function prototypes ]-----------------------*/

#ifdef __cplusplus                     /* No mangling, please           */
   extern   "C"   {
#endif
FlgT  CTYP  NlsIsAlpha(ChrT c);        /* Return TRUE if NLS alphabetic */
FlgT  CTYP  NlsIsLower(ChrT c);        /* Return TRUE if NLS lowercase  */
FlgT  CTYP  NlsIsPrint(ChrT c);        /* Return TRUE if NLS printable  */
FlgT  CTYP  NlsIsUpper(ChrT c);        /* Return TRUE if NLS uppercase  */
ChrT  CTYP  NlsToLower(ChrT c);        /* Convert to NLS lower, if upper*/
ChrT  CTYP  NlsToUpper(ChrT c);        /* Convert to NLS upper, if lower*/
                              /*- Internals ----------------------------*/
WrdT  CTYP  _NlsInit(NOARG);
IntT  PAS   _NlsGet(BytFP c, BytFP u);
#ifdef __cplusplus
   }
#endif
#endif   /*- _TCXLnls_ : End of TCXLnls.h ------------------------------*/
