/*=====[ The TesSeRact(TM) CXL User Interface Development System ]======*
 | Copyright (c) 1987-1991, Innovative Data Concepts, Inc.
 | All Rights Reserved.
 |
 | This Library is part of IDC's TesSeRact Development Tools product
 | line. For information about other IDC products, call 1-215-443-9705.
 *-V--------------------------------------------------------------------*
 | $Header:   tcxlunx.h     552.2   21 Apr 1991 12:53:20  $
 |
 | $Log:   tcxlunx.h    $
 *-D--------------------------------------------------------------------*
 | <TCXLunx.h> : Definition and prototypes for internal TCXL Unix
 | interface functions.
 *-N-----------------------[ Notes and Caveats ]------------------------*
 |
 *======================================================================*/
#ifndef  _TCXLunx_
#  define   _TCXLunx_ 1       /* only once! */
#  ifndef   _TCXLdef_
#     include  <TCXLdef.h>
#  endif

#  include <sys/types.h>
#  include <sys/stat.h>

#  include <dirent.h>

#  if defined(__SCOXENIX__)
/*
   Must define SCO struct here since the d_ino field is defined in
   the SCO dirent.h file as a long and it should be a ino_t
   (which is a short).
*/
typedef struct                         /* data from readdir()           */
   {
   ino_t    d_ino;                     /* inode number of entry         */
   WrdT    unused1;                    /* Hi half of LONG with offset   */
   WrdT    unused2;                    /* Lo half of LONG with offset   */
   unsigned short d_reclen;            /* length of this record         */
   char     d_name[1];                 /* name of file                  */
   } DIRENT;                           /* struct to hold one dir ent    */
#  else
   typedef struct dirent DIRENT;       /* struct to hold one dir ent    */
#  endif

/*------------------[ Data objects and access macros ]------------------*/

#ifndef _D_NAME_MAX
#   define _D_NAME_MAX 255
#endif

TYP   union    _Cax  _CaxT;            /* Object, pointer types         */
TYP   union    _Cdi  _CdiT;
TYP   struct   _Dev  _DevT, *_DevP, FAR *_DevFP;
TYP   struct   _Dtm  _DtmT, *_DtmP;
TYP   struct   _Ffb  _FfbT, *_FfbP;

union    _Cax                 /*-[ Critical error [AX] overlay bitmap ]-*/
{  WrdT     cax;              /*                                        */
   struct                     /*                                        */
   {  BitT  cdrv  : 8;        /* _CaxDrv : Drive code [0=A, 1=B, ...]   */
      BitT  crdw  : 1;        /* _CaxRdw : Read/write                   */
      BitT  cloc  : 2;        /* _CaxLoc : Error Locus                  */
      BitT  cfai  : 1;        /* _CaxFai : FAIL action permitted        */
      BitT  cret  : 1;        /* _CaxRet : RETRY action permitted       */
      BitT  cign  : 1;        /* _CaxIgn : IGNORE action permitted      */
      BitT        : 1;        /*                                        */
      BitT  cdsk  : 1;        /* _CaxDsk : 0 if disk error, else 1      */
  }   cab;                    /*                                        */
};                            /*----------------------------------------*/
#define  _CaxDrv(a)  ((a).cab.cdrv)             /* Drive code           */
#define  _CaxRd(a)   (0 == (a).cab.crdw)        /* Read error           */
#define  _CaxWr(a)   (0 != (a).cab.crdw)        /* Write error          */
#define  _CaxLoc(a)  ((a).cab.cdrv)             /* Error Locus          */
#define  _CaxSys(a)  (0 == _CaxLoc(a))             /* DOS system area   */
#define  _CaxFat(a)  (1 == _CaxLoc(a))             /* FAT area          */
#define  _CaxDir(a)  (2 == _CaxLoc(a))             /* Directory         */
#define  _CaxDat(a)  (3 == _CaxLoc(a))             /* Data area         */
#define  _CaxFai(a)  (0 != (a).cab.cfai)        /* FAIL permitted       */
#define  _CaxRet(a)  (0 != (a).cab.cret)        /* RETRY permitted      */
#define  _CaxIgn(a)  (0 != (a).cab.cign)        /* IGNORE permitted     */
#define  _CaxDsk(a)  (0 == (a).cab.cdsk)        /* Disk error           */


union    _Cdi                 /*-[ Critical error [DI] overlay ]--------*/
{  WrdT     cdi;              /*                                        */
   struct                     /*                                        */
   {  BytT  cerr;             /* _CdiErr : Error code in low half       */
      BytT  cdih;             /*                                        */
   }  clh;                    /*                                        */
};                            /*----------------------------------------*/
#define  _CdiErr(d)  ((d).clh.cerr)             /* Error code           */

struct   _Dev                 /*-[ MS-DOS device-driver header ]--------*/
{  DwdT     dnxt;             /* 00|00 : _DevNxt : Next header          */
   WrdT     datr;             /* 04|04 : _DevAtr : Attributes           */
   WrdT     dstr;             /* 06|06 : _DevStr : Strategy entry       */
   WrdT     dint;             /* 08|08 : _DevInt : Interrupt entry      */
   ChrT     dnam[8];          /* 0A|0A : _DevNam : Device name          */
};                            /* 12|12 ]--------------------------------*/
#define  _DevNxt(d)  ((d)->dnxt)                /* Next header          */
#define  _DevAtr(d)  ((d)->datr)                /* Attributes           */
#define  _DevStr(d)  ((d)->dstr)                /* Strategy entry       */
#define  _DevInt(d)  ((d)->dint)                /* Interrupt entry      */
#define  _DevNam(d)  ((d)->dnam)                /* Device name          */

struct   _Dtm                 /*-[ System date/time object ]------------*/
{  WrdT  dyr;                 /* 00|00 : _DtmYr  : Year (1980-2099)     */
   BytT  dmon;                /* 02|02 : _DtmMon : Month (1=Jan)        */
   BytT  dday;                /* 03|03 : _DtmDay : Day of month (1..31) */
   BytT  ddow;                /* 04|04 : _DtmDow : Day of week (0=Sun)  */
   BytT  dhr;                 /* 05|05 : _DtmHr  : Hour (0..23)         */
   BytT  dmin;                /* 06|06 : _DtmMin : Minute (0..59)       */
   BytT  dsec;                /* 07|07 : _DtmSec : Second (0..59)       */
   BytT  ddec;                /* 08|08 : _DtmDec : Hundredth (0..99)    */
   BytT  dpad;                /* 09|09 : Pad to even word boundary      */
};                            /* 0A|0A ]--------------------------------*/

GBL   _DtmT _DatTim;          /*- Global date/time object --------------*/
#define  _DtmYr   (_DatTim.dyr)                 /* Year                 */
#define  _DtmMon  (_DatTim.dmon)                /* Month                */
#define  _DtmDay  (_DatTim.dday)                /* Day of month         */
#define  _DtmDow  (_DatTim.ddow)                /* Day of week          */
#define  _DtmHr   (_DatTim.dhr)                 /* Hour                 */
#define  _DtmMin  (_DatTim.dmin)                /* Minute               */
#define  _DtmSec  (_DatTim.dsec)                /* Second               */
#define  _DtmDec  (_DatTim.ddec)                /* Hundredth            */

/*------------------[ Include DOS compatible bits ]---------------------*/
#define  FA_RDO      0x01     /* read only attribute  */
#define  FA_HID      0x02     /* hidden               */
#define  FA_SYS      0x04     /* system               */
#define  FA_VOL      0x08     /* volume label         */
#define  FA_DIR      0x10     /* directory            */
#define  FA_ARC      0x20     /* archive              */
#define  FA_DEV      0x40     /* device *UNDOC*       */

/*-----------------[ UNIX file type bits ]------------------------*/

#define FA_FILTYP 0xf000
#define FA_NAMPIP 0x1000
#define FA_CHRSPC 0x2000
#define FA_DIRECT 0x4000
#define FA_NAMSPC 0x5000
#define FA_BLKSPC 0x6000
#define FA_NORMAL 0x8000

#define FA_SETUID 0x0800
#define FA_SETGID 0x0400
#define FA_SAVTXT 0x0200

#define FA_OWNRDA 0x0100
#define FA_OWNWTA 0x0080
#define FA_OWNEXA 0x0040
#define FA_OWNACC (FA_OWNRDA | FA_OWNWTA | FA_OWNEXA)

#define FA_GRPRDA 0x0020
#define FA_GRPWTA 0x0010
#define FA_GRPEXA 0x0008
#define FA_GRPACC (FA_GRPRDA | FA_GRPWTA | FA_GRPEXA)

#define FA_WLDRDA 0x0004
#define FA_WLDWTA 0x0002
#define FA_WLDEXA 0x0001
#define FA_WLDACC (FA_WLDRDA | FA_WLDWTA | FA_WLDEXA)

/*-----------------[ UNIX _Ffb definition ]------------------------*/

struct _Ffb                        /* find first/find next structutre   */
{  BytT  fdos[21];                 /* 00|00 : _FfbDos : DOS reserved    */
   BytT  fatr;                     /* dos compatible file attribute     */
   WrdT  ftim;                     /* dos compatible file time          */
   WrdT  fdat;                     /* dos compatible file date          */
   ChrT  fnam[15];                 /* first 14 chars of path            */
   BytT  fiat;                     /* internal attribute used by FndNxt */
   ChrT  fdrv;                     /* dos compatible drive letter       */
   ChrP  fpth;                     /* name of dir open                  */
   ChrP  fmsk;                     /* file mask to search for           */
   ChrP  fpfl;                     /* pointer to where file name goes   */
   DIR *fhnd;                      /* pointer to open directory struct  */
   DIRENT *fent;                   /* pointer to dirent struct          */
   struct stat fsta;               /* stat() return buffer              */
};

/*-----------------------[ DOS compatible access macros ]------------*/
#define  _FfbDos(f)  ((f).fdos)                       /* DOS-reserved   */
#define  _FfbAtr(f)   ((f).fatr)                      /* file attribute */
#define  _FfbRdo(f)   (0 != ((f).fatr & FA_RDO))      /*    read-only   */
#define  _FfbHid(f)   (0 != ((f).fatr & FA_HID))      /*    hidden      */
#define  _FfbSys(f)   (0 != ((f).fatr & FA_SYS))      /*    system      */
#define  _FfbVol(f)   (0 != ((f).fatr & FA_VOL))      /*    volume      */
#define  _FfbDir(f)   (0 != ((f).fatr & FA_DIR))      /*    directory   */
#define  _FfbArc(f)   (0 != ((f).fatr & FA_ARC))      /*    archive     */
#define  _FfbDev(f)   (0 != ((f).fatr & FA_DEV))      /*    device      */
#define  _FfbTim(f)   ((f).ftim)                      /* packed time    */
#define  _FfbDat(f)   ((f).fdat)                      /* packed date    */
#define  _FfbNam(f)   ((f).fnam)                      /* file name      */
#define  _FfbDot(f)   ((f).fnam[0]=='.')              /*    '.' or '..' */
#define  _FfbPar(f)   (_FfbDot(f) && ((f).fnam[1] == '.')) /* parent dir*/

/*---------------------[ UNIX access macros ]-------------------------*/
#define  _FfbIat(f)   ((f).fiat)                    /* int attribute    */
#define  _FfbPfl(f)   ((f).fpfl)                    /* pointer to file  */
#define  _FfbDrv(f)   ((f).fdrv)                    /* dos drive letter */
#define  _FfbPth(f)   ((f).fpth)                    /* name of dir      */
#define  _FfbMsk(f)   ((f).fmsk)                    /* mask to match    */
#define  _FfbHnd(f)   ((f).fhnd)                    /* open dir handle  */
#define  _FfbEnt(f)   ((f).fent)                    /* curr file data   */
#define  _FfbRle(f)   (_FfbEnt(f)->d_reclen)        /* record length    */
#define  _FfbNle(f)   (_FfbEnt(f)->d_namlen)        /* name   length    */
#define  _FfbUnm(f)   (_FfbEnt(f)->d_name)          /* name of file     */

#define  _FfbSta(f)   ((f).fsta)                    /* status info      */
#define  _FfbBit(f)   (_FfbSta(f).st_mode)          /* file mode bit    */
#define  _FfbIno(f)   (_FfbSta(f).st_ino)           /* inode number     */
#define  _FfbDvi(f)   (_FfbSta(f).st_dev)           /* ID of device     */
#define  _FfbRdv(f)   (_FfbSta(f).st_rdev)          /* ID of dev (special)*/
#define  _FfbLnk(f)   (_FfbSta(f).st_nlink)         /* number of links  */
#define  _FfbUid(f)   (_FfbSta(f).st_uid)           /* user id of file  */
#define  _FfbGid(f)   (_FfbSta(f).st_gid)           /* group id of file */
#define  _FfbSiz(f)   (_FfbSta(f).st_size)          /* size of file     */
#define  _FfbAcc(f)   (_FfbSta(f).st_atime)         /* last access time */
#define  _FfbMod(f)   (_FfbSta(f).st_ftime)         /* data mod time    */
#define  _FfbChg(f)   (_FfbSta(f).st_ctime)         /* stat mod time    */

#define  _FfbFty(f)   (0 != (_FfbBit(f) & FA_FILTYP))
#define  _FfbPip(f)   ((_FfbBit(f) & FA_FILTYP) == (FA_NAMPIP))
#define  _FfbCsp(f)   ((_FfbBit(f) & FA_FILTYP) == (FA_CHRSPC))
#define  _FfbDct(f)   ((_FfbBit(f) & FA_FILTYP) == (FA_DIRECT))
#define  _FfbNsp(f)   ((_FfbBit(f) & FA_FILTYP) == (FA_NAMSPC))
#define  _FfbBsp(f)   ((_FfbBit(f) & FA_FILTYP) == (FA_BLKSPC))
#define  _FfbNor(f)   ((_FfbBit(f) & FA_FILTYP) == (FA_NORMAL))

#define  _FfbSui(f)   (0 != (_FfbBit(f) & FA_SETUID))
#define  _FfbSgi(f)   (0 != (_FfbBit(f) & FA_SETGID))
#define  _FfbTxt(f)   (0 != (_FfbBit(f) & FA_SAVTXT))

#define  _FfbOrd(f)   (0 != (_FfbBit(f) & FA_OWNRDA))
#define  _FfbOwt(f)   (0 != (_FfbBit(f) & FA_OWNWTA))
#define  _FfbOex(f)   (0 != (_FfbBit(f) & FA_OWNEXA))
#define  _FfbOac(f)   (0 != (_FfbBit(f) & FA_OWNACC))

#define  _FfbGrd(f)   (0 != (_FfbBit(f) & FA_GRPRDA))
#define  _FfbGwt(f)   (0 != (_FfbBit(f) & FA_GRPWTA))
#define  _FfbGex(f)   (0 != (_FfbBit(f) & FA_GRPEXA))
#define  _FfbGac(f)   (0 != (_FfbBit(f) & FA_GRPACC))

#define  _FfbWrd(f)   (0 != (_FfbBit(f) & FA_WLDRDA))
#define  _FfbWwt(f)   (0 != (_FfbBit(f) & FA_WLDWTA))
#define  _FfbWex(f)   (0 != (_FfbBit(f) & FA_WLDEXA))
#define  _FfbWac(f)   (0 != (_FfbBit(f) & FA_WLDACC))

/*------------------------[ Function prototypes ]-----------------------*/

                                       /* Critical error handler        */
TYP   IntT  (FAR CTYP *IffpCP)(WrdT, WrdT, WrdT, WrdT);

#ifdef   __cplusplus                   /* No mangling, please           */
   #define  _IFFPCP(f)   (IffpCP)(f)   /* crufty cast for C++           */
   extern   "C"   {
#else
   #define  _IFFPCP(f)   (f)           /* no crufty cast for C          */
#endif
                                       /*- Critical error handling -----*/
VOID        _CerrOut(NOARG);           /* Uninstall TCXL CritErr trap   */
IntT        _CerrPush(IffpCP fa);      /* Push new CritErr handler      */
IntT        _CerrPop(NOARG);           /* Pop last CritErr handler      */
IntT        _CerrSet(IffpCP fa);       /* Overwrite current handler     */
IffpCP      _CerrGet(NOARG);           /* Get last CritErr handler addr */
IntT  FAR   CTYP  _DefCrit(WrdT rsi,   /* Default handler               */
   WrdT rbp, WrdT rdi, WrdT rax);
IntT  FAR   CTYP  _AltCrit(WrdT rsi,   /* Alternate (verbose) handler   */
   WrdT rbp, WrdT rdi, WrdT rax);
                                       /*- MS-DOS interface ------------*/
FlgT  PAS   _ChgDir(ChrP p);           /* Change directory              */
ChrT  PAS   _ChgDrv(ChrT d);           /* Change current drive          */
IntT  CTYP  _FilFnd(ChrP p, ChrP b);   /* Find file along PATH          */
FlgT  PAS   _FndFst(ChrP p, _FfbP f,   /* Find first file               */
   IntT a);
FlgT  PAS   _FndNxt(_FfbP f);          /* Find next file                */
IntT  PAS   _GetDir(ChrT d, ChrP p);   /* Get current dir               */
VOID  PAS   _GetDtm(NOARG);            /* Get system date/time          */
ChrT  PAS   _GetDrv(NOARG);            /* Get current drive             */
WrdT  PAS   _GetSys(NOARG);            /* Get system version            */
                                       /*- iApx86 interface ------------*/
BytT  PAS   _InpByt(WrdT p);           /* Input byte                    */
WrdT  PAS   _InpWrd(WrdT p);           /* Input word                    */
VOID  PAS   _IntDis(NOARG);            /* Disable interrupts            */
VOID  PAS   _IntEna(NOARG);            /* Enable interrupts             */
VOID  PAS   _OutByt(WrdT p, BytT b);   /* Output byte                   */
VOID  PAS   _OutWrd(WrdT p, WrdT w);   /* Output word                   */

ChrP regcmp(ChrP str1, ...);                    /* compile reg exp      */
ChrP regex(ChrP pat, ChrP subj, ...);           /* compare reg exp      */

VOID  PAS    _TcxlSigHang(IntT sig);    /* Catch SIGHUP  routine        */
VOID  PAS    _TcxlSigIntr(IntT sig);    /* Catch SIGINT  routine        */
VOID  PAS    _TcxlSigIo(IntT sig);      /* Catch SIGIO   routine        */
VOID  PAS    _TcxlSigQuit(IntT sig);    /* Catch SIGQUIT routine        */
VOID  PAS    _TcxlSigTerm(IntT sig);    /* Catch SIGTERM routine        */

ChrP  PAS    itoa(IntT val, ChrP str, IntT rad); /* need declaration    */

#define _CerrOut()                     /* has no meaning in UNIX        */
#define _CerrPush(a) (0)               /* has no meaning in UNIX        */
#define _CerrPop() (-1)                /* has no meaning in UNIX        */
#define _CerrSet(a) (-1)               /* has no meaning in UNIX        */
#define _CerrGet() (NULL)              /* has no meaning in UNIX        */

#ifdef __cplusplus
   }
#endif
#endif   /*- _TCXLdos_ : End of TCXLdos.h ------------------------------*/
